const editor = document.querySelector('.article-editor');
const options = {
  placeholder: '请输入正文',
  scrollingContainer: document.documentElement
};
const quill = new Quill(editor, options);

/**
 * 注册编辑器分割线
 */
const BlockEmbed = Quill.import('blots/block/embed');
const Link = Quill.import('formats/link');

class DividerBlot extends BlockEmbed { }

// 分割线
DividerBlot.blotName = 'divider';
DividerBlot.tagName = 'hr';

Quill.register(DividerBlot);

class HhImageBlot extends BlockEmbed {
  static create(value) {
    let wrapNode = super.create();
    let img = document.createElement('img');
    wrapNode.setAttribute('class', 'editor-image-wrap');
    wrapNode.setAttribute('contenteditable', 'false');
    img.setAttribute('src', value);
    wrapNode.appendChild(img);
    return wrapNode;
  }

  static value(domNode) {
    if (domNode.querySelector('img')) {
      return domNode.querySelector('img').getAttribute('src');
    }
  }

  static formats(domNode) {
    return ['alt', 'height', 'width', 'class'].reduce(function(formats, attribute) {
      if (domNode.hasAttribute(attribute)) {
        formats[attribute] = domNode.getAttribute(attribute);
      }
      return formats;
    }, {});
  }

  static match(url) {
    return /\.(jpe?g|gif|png)$/.test(url) || /^data:image\/.+;base64/.test(url);
  }

  static sanitize(url) {
    return Link.sanitize(url, ['http', 'https', 'data']) ? url : '//:0';
  }

  format(name, value) {
    if (['alt', 'height', 'width', 'class'].indexOf(name) > -1) {
      if (value) {
        this.domNode.setAttribute(name, value);
      } else {
        this.domNode.removeAttribute(name);
      }
    } else {
      super.format(name, value);
    }
  }
}

HhImageBlot.blotName = 'hhImage';
HhImageBlot.tagName = 'DIV';

Quill.register(HhImageBlot);

/**
 * title相关事件
 */
const titleTextareaContent = document.querySelector('.title-textarea-content');
const titleTextarea = document.querySelector('.title-textarea');
const titleError = document.querySelector('.title-error-message');

// 监听标题输入框输入事件
titleTextarea.addEventListener('input', (event) => {
  // 清空标题错误提示
  titleError.innerHTML = '';

  titleTextarea.value = event.target.value;
  titleTextareaContent.innerHTML = event.target.value;
  callWebviewMethod('jkttTitleChange', event.target.value);
})

titleTextarea.addEventListener('blur', () => {
  const length = titleTextarea.value.length
  if (length < 5) {
    titleError.innerHTML = '标题字数不可少于5个字';
  }
  callWebviewMethod('jkttTitleBlur', null);
})

titleTextarea.addEventListener('focus', () => {
  callWebviewMethod('jkttTitleFocus', null);
})

/**
 * 编辑器change事件
 */
quill.on('text-change', (delta, oldDelta, source) => {
  let html = editor.children[0].innerHTML;
  const text = quill.getText();

  if (html === '<p><br></p>') html = '';

  const dataMap = {
    text,
    html,
    history: quill.history.stack,
    format: quill.getFormat()
  }

  callWebviewMethod('jkttContentChange', JSON.stringify(dataMap));
});

/**
 * 编辑器光标改变事件
 */
quill.on('selection-change', (range, oldRange, source) => {
  if (range) {
    callWebviewMethod('jkttEditorFocus', quill.getFormat())
  } else {
    callWebviewMethod('jkttEditorBlur')
  }
});

// respond to clicks inside the editor
quill.root.addEventListener('click', this.handleClick, false);

// 编辑器格式化内容
window.formatText = (name, value) => {
  quill.format(name, value);
}

// 插入图片
window.insertImage = (urls) => {
  // 获取光标所在位置
  const length = quill.getSelection(true).index;
  const images = urls.split(',');

  images.map((item, index) => {
    // 插入图片
    quill.insertEmbed(length + index, 'hhImage', item);
    // quill.insertEmbed(length + index, 'image', item);
  })

  // 调整光标到最后
  quill.setSelection(length + images.length);
}

// 插入分割线
window.insertDivider = () => {
  const length = quill.getSelection(true).index;
  quill.insertEmbed(length, 'divider', true);
  quill.setSelection(length + 1);
}

// 撤销/恢复
window.reOrUndo = (type) => {
  if (type === '1') {
    quill.history.redo();
  } else {
    quill.history.undo();
  }
}

// 插入emoji
window.insertEmoji = (unicode) => {
  const length = quill.getSelection(true).index;
  quill.insertText(length, unicode);
}

// 删除文本
window.deleteText = () => {
  const length = quill.getSelection(true).index;
  quill.deleteText(length - 1, 1);
}

// 设置平台
window.setPlatform = function (platform) {
  window.platform = platform;
}

// 设置数据
window.setData = function (title, textContent) {
  titleTextarea.value = title;
  quill.pasteHTML(textContent);
}

// const btn = document.querySelector('.setting-btn');
// const btn1 = document.querySelector('.setting-btn1');

// btn.addEventListener('click', () => {
//   // window.insertDivider();
//   // console.log(quill.history.stack);
//   // window.reOrUndo('1')
//   // quill.format('bold', true)
//   // quill.format('header', '1')
//   // window.setData('标题标题', '<div class="editor-image-wrap is-cover" contenteditable="false"><img src="https://hhhealthapp.oss-cn-beijing.aliyuncs.com/20210114/7fd69a20-dc2d-403a-b3fc-1b73e4d0cbe11858.jpg"></div><div class="editor-image-wrap " contenteditable="false"><img src="https://hhhealthapp.oss-cn-beijing.aliyuncs.com/20201231/4c8ec94e-ba07-4f23-86f2-dc1fe4ed22558687.jpg"></div><p><br></p>')
//   // window.setData('标题标题', '<img src="https://hhhealthapp.oss-cn-beijing.aliyuncs.com/20210114/7fd69a20-dc2d-403a-b3fc-1b73e4d0cbe11858.jpg">')
//   window.setData('标题标题', `<p>据台湾《中国时报》报道，由于“解放军频繁扰台、台海局势升温”，台湾“海巡署”日前公告，3月1日将在东沙岛实施火炮实弹射击，模拟解放军袭扰等事件进行演训。与此同时，解放军也在南海区域展开为期一个月的演练。两岸同期进行演练是巧合吗？
//   </p><p>
//   </p><p>东沙岛实弹射击引关注
//   </p><p>
//   </p><p>台湾“新新闻”日前报道称，根据射击通报内容，东沙岛附近海域演训时间从3月1日持续至9日，射击范围由东沙岛向外海延伸8海里，最大弹道高度为1.2万米。《中国时报》报道称，台湾“海巡署”自2000年成立，接替原本驻守岛上的海军陆战队，驻守东沙岛、太平岛，两处防御主力虽为“海巡署”官兵，但经台军陆战队训练，因此本质上也具有基本陆战队战力。至于这次演练是否动用岛上配备的120毫米迫击炮、81毫米迫击炮、40毫米防空炮、40毫米榴弹发射器、T-75机枪与20机炮等火炮武器，“海巡署”则不愿透露。
//   </p><p>
//   </p><p>台湾“海巡署”强调，东沙岛的对海实弹射击是例行性火炮训练，每季1次，由“海巡署”东沙分署实施射击训练，军方并未参与演训。
//   </p><p>
//   </p><p>但“新新闻”的报道认为，去年曾有境外媒体报道称，相比直接攻打台湾，解放军未来更有可能先夺取台湾所控制的东沙群岛，迄今夺岛作战阴影仍在。该文章称，“海巡署”日前亦由海军中将退伍的周美伍接掌，强化战备用意明显。
//   </p><p>
//   </p><p>大陆军事专家宋忠平1日对《环球时报》记者表示，无论是台海军陆战队还是“海巡署”的战力都很有限，台军所谓的抗登陆作战防夺岛能力，完全就是政治作秀，根本不具备实际作战能力，其防御能力也完全不堪一击。
//   </p><p>
//   </p><p>宋忠平表示，“台军在东沙岛配备的武器很多都属于轻武器，这些武器在岛屿防御作战中难以发挥太大作用。夺岛作战必然是立体作战，包括制空权夺取、实施空地打击以及两栖作战能力，靠这些威力有限的武器装备以及近射程迫击炮来实施岛屿作战，战力有限。”
//   </p><p>
//   </p><p>两岸同日开始演练是巧合吗
//   </p><p>
//   </p><p>巧合的是，广东海事局日前发布的航行警告显示，从3月1日至31日南海雷州半岛西部海域将有军事训练，演习时间长达一个月。台媒对此高度关注。
//   </p><p>
//   </p><p>大陆官方并没有对外透露此次演习细节，但根据央视的报道，近期解放军演习不仅是在南海舰队，北海舰队、东海舰队都在同时展开。尤其值得注意的是，春节过后南部战区海军航空兵某轰炸机团新老飞行员齐上演训场，展开对海突击、对海打击等课目训练，以老带新检验协同作战能力。近日东部战区海军航空兵某旅在东海某海域组织大机群对海突击演练，预警机、歼轰机同时升空，检验飞行员空中协同打击能力。
//   </p><p>
//   </p><p>近一段时期台海局势升温，解放军常态化派遣军机进入台湾周边空域巡航。新加坡《联合早报》报道称，自去年以来，解放军东部战区和南部战区的歼-16战机经常出动大机队，在台海空域进行战备演练。在2月20日晚间，台湾军方曾通报称，两架次歼-10、两架次歼-16、4架次歼轰-7、两架次轰-6K、1架次运-8反潜机进入台湾西南空域。
//   </p><p>
//   </p><p>大陆会夺岛吗？美前防长这样说
//   </p><p>
//   </p><p>不仅台媒在关注和炒作有关“大陆夺岛风险”的话题，曾担任美防长的盖茨2月26日参加《华盛顿邮报》一场线上活动时表示，北京可能会考虑夺取接近大陆、由台湾实际控制的岛屿，这可能会是一种蚕食战略。
//   </p><p>
//   </p><p>近期一些分析认为，北京的目标将是东沙群岛。盖茨警告，台海局势非常危险，在美中多面向竞争中，他最为担心台湾局势。随着中国大陆军事实力增强，北京可能对台采取行动，或是认为自己能在意料之外的对抗中获胜。
//   </p><p>
//   </p><p>对于美国的应对办法，盖茨建议，美国应认真考虑，是时候放弃在台湾议题上长期以来的“战略模糊”，明确告诉两岸，不要单方面动武或寻求改变现状。“告诉中国（大陆）人，若无缘无故对台湾采取行动，美国将会支持台湾。”盖茨说，“同时告诉台湾，若单方面改变现状，就得靠自己。”
//   </p><p>
//   </p><p>外交学院教授李海东1日对《环球时报》表示，盖茨的言论表明美国是不会放弃对中国内政事务干预的，甚至提及不惜用武力方式进行干预，这种做法会给中美关系带来巨大波折和动荡，会为台海局势造成恶劣影响。“盖茨的言论是损人害己的主张。”
//   </p><p>
//   </p><p>1日，中国国防部新闻局就近期热点问题答记者问，其中在回应中美两军军事关系时表示，“当前中美关系正处于重要关口，中美合则两利、斗则俱伤，合作是双方唯一正确选择。”
//   </p><p>
//   </p><p>一位匿名人士1日对《环球时报》记者表示，外界关于解放军解决台湾问题将先夺取离岛只是一种猜测。在未来解放军决定武力统一台湾的时候必然是民进党当局触犯《反分裂国家法》所划下的法律红线，在这种情况下解决台湾问题不需要一个小岛一个小岛逐步解决，而是会通盘考虑，一次性解决。“解放军目前已经具备了登岛作战能力，用最小的代价，最短时间彻底解决台湾问题，不给‘台独’去争取域外势力帮助留下空间，不给‘台独’势力喘息机会。”▲</p><p><br></p><p><br></p><div class="editor-image-wrap" contenteditable="false"><img src="http://hhhealthapp.oss-cn-beijing.aliyuncs.com/20210302/1614651615986.jpeg"></div><p><br></p>`)
//   // window.insertEmoji('\uD83D\uDE03')
// })

// btn1.addEventListener('click', () => {
//   // quill.format('list', 'bullet')
//   // const length = quill.getSelection(true).index;
//   // quill.deleteText(length - 1, 1)
//   // window.reOrUndo('0')
//   // let html = editor.children[0].innerHTML;
//   // const text = quill.getText();

//   // if (html === '<p><br></p>') html = '';
//   // const dataMap = {
//   //   text,
//   //   html,
//   //   history: quill.history.stack,
//   //   format: quill.getFormat()
//   // }
//   // callWebviewMethod('jkttContentChange', JSON.stringify(dataMap));
//   window.insertImage('https://hhhealthapp.oss-cn-beijing.aliyuncs.com/20210114/7fd69a20-dc2d-403a-b3fc-1b73e4d0cbe11858.jpg,https://hhhealthapp.oss-cn-beijing.aliyuncs.com/20201231/4c8ec94e-ba07-4f23-86f2-dc1fe4ed22558687.jpg');
// })

let focusImage = '';
let selectedFocusImage;

// 处理图片点击
function handleClick(evt) {
  if (evt.target && evt.target.tagName && evt.target.tagName.toUpperCase() === 'IMG') {
    if (focusImage === evt.target) {
      // we are already focused on this image
      return;
    }
    if (focusImage) {
      // we were just focused on another image
      hideImageSelect();
    }
    // clicked on an image inside the editor
    let select = document.createElement('div');
    select.setAttribute('class', 'editor-image-select');
    select.innerHTML = hasClass(evt.target.parentNode, 'is-cover') ? '取消封面' : '设为封面';
    focusImage = evt.target;
    focusImage.parentNode.appendChild(select);
    
    addClass(evt.target.parentNode, 'editor-image-border');
    document.querySelector('.editor-image-select').addEventListener('click', handleImageSelectClick);
    setTimeout(() => {
      quill.blur();
    });
  } else if (focusImage) {
    // clicked on a non image
    // this.hide();
    hideImageSelect();
  }
}

function handleImageSelectClick(e) {
  e.stopPropagation();
  const imgParent = focusImage.parentNode;

  if (hasClass(imgParent, 'is-cover')) {
    removeClass(imgParent, 'is-cover');
    selectedFocusImage = '';
  } else {
    const isCover = document.querySelectorAll('.editor-image-wrap.is-cover');
    isCover.forEach((item) => {
      removeClass(item, 'is-cover');
    })
    addClass(imgParent, 'is-cover');
    selectedFocusImage = focusImage.src;
  }
  callWebviewMethod('jkttSetOrCancelCover', selectedFocusImage);
  hideImageSelect();
}

function hideImageSelect() {
  removeClass(focusImage.parentNode, 'editor-image-border');
  document.querySelector('.editor-image-select').removeEventListener('click', handleImageSelectClick);
  focusImage.parentNode.removeChild(document.querySelector('.editor-image-select'));
  focusImage = undefined;
}

function callWebviewMethod(name, value) {
  if (window.platform) {
    if (window.platform.toLowerCase() === 'ios') {
      window.webkit.messageHandlers[name].postMessage(value);
    } else {
      jktt[name](value ? value : '');
    }
  }
}

function addClass(ele, cls) {
  if (!hasClass(ele, cls)) ele.className += ' ' + cls
}

function removeClass(ele, cls) {
  if (hasClass(ele, cls)) {
    const reg = new RegExp('(\\s|^)' + cls + '(\\s|$)')
    ele.className = ele.className.replace(reg, ' ')
  }
}

function hasClass(ele, cls) {
  return !!ele.className.match(new RegExp('(\\s|^)' + cls + '(\\s|$)'))
}
